# Copyright 2010, <jim.foltz@gmail.com>
#
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------

# Version: 2.0
# Date: 2014-05-18
# Compatible with Sketchup 2014

module JF

  module SaveAsPrevious2

    VERSION = '2.0'.freeze
    PREF_KEY = 'JF\SaveAsPrevious2'.freeze
    RESTRICTED_CHARS = '<>:"/\|?*'


    def self.save_as_previous_version(version)
      if Sketchup.active_model.title.empty?
        UI.messagebox('Model must be saved first.')
        return
      end
      format    = read_format()
      save_name = sub_format(version, format)
      if Sketchup.version.to_i < 14 
        UI.messagebox("This plugins does not\nsupport SketchUp versions\nbefore 2014")
      else
        save_as_prev_14(version, save_name)
      end
    end

    def self.model_name
      path = Sketchup.active_model.path
      name = File.basename(path, '.skp')
    end

    def self.read_format
      format = Sketchup.read_default(PREF_KEY, 'format', '%model_v%version_%F')
    end

    def self.write_format(format)
      Sketchup.write_default(PREF_KEY, 'format', format)
    end

    def self.do_format()
      format = read_format
      ret = UI.inputbox( ["Format         "], [format],[], "Edit Format" )
      return if ret == false
      format = ret[0].strip
      return unless check_format(format)
      if confirm_format(ret[0].dup) == IDYES
        write_format(ret[0])
        UI.messagebox("Format saved.")
      end
    end

    def self.check_format(format)
      st = format.delete!(RESTRICTED_CHARS)
      if st
        UI.messagebox("Restricted chars removed:#{RESTRICTED_CHARS.inspect}")
      end
      f = sub_format('{VER}', format.dup)
      if f.length == 0
        UI.messagebox("Format resulted in 0 length filename.\n Try again.")
        return false
      end
      return true
    end

    def self.confirm_format(format)
      f = sub_format('{VER}', format.dup)
      UI.messagebox("Model name:#{model_name()}\nFormat: #{format.inspect}\n\nModel would be saved as:\n#{f}.skp\n\nWould you like to save this\sas the default format?", MB_YESNOCANCEL)
    end

    def self.sub_format(version, format)
      format.gsub!(/%model/, model_name())
      format.gsub!(/%version/, version.to_s)
      save_name = Time.now.strftime(format)
      save_name
    end

    def self.check_overwrite(path)
      r = true
      if File.exists?(path)
        r = UI.messagebox("File #{path} exists.\n\nOverwrite?", MB_YESNO) == IDYES
      end
      return r
    end

    def self.save_as_prev_14(version, save_name)
      version += 2000 if version > 8
      ver = const_get("Sketchup::Model::VERSION_#{version}")
      path = Sketchup.active_model.path
      if path.empty?
        UI.messagebox("Please save your model first.")
      else
        dirname = File.dirname(path)
        sname   = File.join(dirname, save_name)
        sname   = sname.tr('\\', '/')
        sname << ".skp"
        return unless check_overwrite(sname)
        st = Sketchup.active_model.save_copy(sname, ver)
        if st
          UI.messagebox("Copy saved as: #{sname}")
        else
          UI.messagbox("A save error occured.")
        end
      end
    end

    def self.display_help()
      msg = "Formats Help\n\n"
      msg << "Dates:\n"
      msg << "\t%Y - Year (#{Time.now.strftime('%Y')})\n"
      msg << "\t%m - Month (#{Time.now.strftime('%m')})\n"
      msg << "\t%d - Day (#{Time.now.strftime('%d')})\n"
      msg << "\n\n"
      msg << "Time:\n"
      msg << "\t%H - Hour (06)\n"
      msg << "\t%M - Minute (29)\n"
      msg << "\t%S - Second (44)\n"
      msg << "\n\n"
      msg << "Model name and Version:\n"
      msg << "\t%model - Model Name"
      msg << "\t%version - SketchUp Version\n"
      msg << "\n\n"
      msg << "Example: \"%model_v%version_%Y-%m-%d\" might be saved as\n"
      msg << "Cabinet_v14_2014-04-09.skp"
      UI.messagebox(msg, MB_OK)
    end

    this_version = Sketchup.version.to_i
    versions = [3, 4, 5, 6, 7, 8]
    versions << (13..(Time.now.year-2000+1)).to_a
    versions.flatten!
    menus = versions[0..(versions.index(this_version))]
    menu = UI.menu('File').add_submenu("[JF] Save Copy As ...")
    menus.reverse.each do |v|
      menu.add_item("Version #{v}") { JF::SaveAsPrevious2.save_as_previous_version(v)}
    end
    menu.add_separator()
    menu.add_item("Edit Format") { do_format() }
    menu.add_item("Version #{VERSION} Help") { display_help() }

  end # module SaveAsPrevious
end # module JF
